options ls=71;
title 'GLS - Prismatolaimus/m2- exponential model' ;
data wetland;
    infile 'prisnema dat a';
    input x y time pris;
    /* if time=1 then delete;
       if time=108 then delete;
       if time=172 then delete;
       if time=290 then delete; */
    prism= pris *2583.4745;   /*converts abundance to a per-m^2 basis */
    run;

/* part 1: defining IML module "vmat", which estimates the spatial correlation matrix "v[i,j]" */

proc iml;
  start vmat;
     v=j(nrow(x),nrow(x),0);
      do i=1 to nrow(x);
           do j=1 to nrow(x);
               dist=sqrt((x[i]-x[j])**2+(y[i]-y[j])**2);
               dtime=abs(t[i]-t[j]);
               dist=dist+10000*dtime;
               if dist<=maxdist then do;
                 if i^=j then v[i,j]=(ce/(co+ce))*exp(-alpha*dist);
                 else v[i,j]=1.0;
               end;
           end;
      end;
  /* print v */
  finish;

/* part 2: Defining the main program "gls", which performs the actual GLS estimation, using the
subroutine "vmat" to calculate the correlation matrix */

  start gls;
     run vmat;
     one=j(nrow(x),1,1);
     xmat=one||xmat;
     vinv=inv(v);
     beta=inv(xmat`*vinv*xmat)*xmat`*vinv*z;
     resid=z-xmat*beta;
     sigma2=resid`*vinv*resid/(nrow(xmat)-ncol(xmat)-1);
     varbeta=sigma2*inv(xmat`*vinv*xmat);
     print beta;
     print sigma2;
     print varbeta;
  finish;

/* part 3: defining data set and parameter values; Actual execution ("run gls") of part 2*/

  use wetland;
     read all var{x} into x;
     read all var{y} into y;
     read all var{time} into t;
     read all var{x} into xmat; /* add "y" between the {...}if y-coordinate is to be tested */
     read all var{prism} into z;
     co=3179512663;    /* best fitting value from "semivari.lst" */
     ce=6357843246;    /* best fitting value from "semivari.lst" */
     alpha=0.057475;    /* best fitting value from "semivari.lst" */
     maxdist=85.;
  run gls;

