options ls=71;
data wetland;
infile 'prisnema dat a';
input x y time pris;
prism= 2583.4745*pris;    /* converts abundances to a per-m^2 basis */
run;

/* part 1: running OLS regression to remove large-scale spatial trend in the data */

     proc reg;
     model   prism=x;   
     output out=a r=resid;
run;

/* part 2: defining IML module "vgram", which estimates semivariances */

     proc iml;
     create vargram var{dist vario n};
     start vgram;  /*  variogram prismatolaimus */
         v=j(nclass,1,0);
         count=j(nclass,1,0);
         do i=1 to nrow(x)-1;
               do j=i+1 to nrow(x);
                    dist=sqrt((x[i]-x[j])**2+(y[i]-y[j])**2);
                    dtime=abs(t[i]-t[j]);
                    dist=dist+10000*dtime;
                    n=int(dist/size)+1;
                    if n<=nclass then do;
                         v[n]=v[n]+(z[i]-z[j])**2;
                         count[n]=count[n]+1;
                    end;
               end;
          end;
          do i=1 to nclass;
               dist=(i-0.5)*size;
               vario=v[i]/(2*count[i]);
               n=count[i];
               if dist<=maxdist then append;
          end;
   finish;

/* part 3: defining data set and parameter values; Actual execution ("run vgram") of part 2*/

   use a;
       read all var{x} into x;
       read all var{y} into y;
       read all var{time} into t;
       read all var{resid}  into z;
       size=10.;
       nclass=9.;
       maxdist=85.;
       run vgram;

/* print semivariances, and plotting the semivariogram: */

proc print data=vargram;
     run;
proc  plot;
     plot vario*dist / vaxis=0,2500000000,5000000000,7500000000,10000000000
                       haxis=0,20,40,60,80,100;
     run;

/* part 4: Fitting an exponential model to the semivariogram using proc nlin: */

proc nlin method=dud;
     parameters  co=3157300000 ce=6388500000 alpha=0.06;
     bounds  co>=0,ce>0, alpha>0;
     temp=co+ce*(1-exp(-alpha*dist));
     model vario=temp;
     _weight_=n/(temp*temp);
     output out=a p=vhat r=resid;
     run;

proc print;
     run;
proc plot;
     plot resid*dist;
     run;