options ls=71;
title 'universal kriging for Prismatolaimus ';

/* Part 1: Data input step, for t=1 only */

data wetland;
infile 'prisnema dat a';
input x y time pris;
   if time=108 then delete;
   if time=172 then delete;
   if time=290 then delete;
   prism=pris*2583.4745;    /* Converts abundances to a per-m^2 basis */
run;

/* Part 2: Creating the data set "points", the set of spatial points for which nematode abundance is
to be predicted */

data points;
     do x0=0 to 48 by 4;
          do y0=0 to 108 by 4;
              output;
          end;
      end;

/* Part 3: Defining IML module "gam1", which estimates semivariances between pairs of
SAMPLED locations "(x,y)", and constructs the "bgamma" matrix" */

proc iml;
    create pred var{x0 y0 zhat};
    start gam1;
         do i=1 to nrow(x);
              do j=1 to nrow(x);
                   dist=sqrt((x[i]-x[j])**2+(y[i]-y[j])**2);
                       if dist=0 then bgamma[i,j]=0;
                       else bgamma[i,j]=(c0+ce*(1-exp(-alpha*dist)));
               end;
          end;
          zero=j(ncol(xmat),ncol(xmat),0);
          bgamma=(bgamma||xmat)//((xmat`)||zero);
    finish;

/* Part 4: Defining IML module "gam2", which estimates semivariances between sampled
locations "(x,y)" and the locations "(xp,yp)" for which predictions are to be made; It constructs
the "sgamma" matrix */

    start gam2;
        do j=1 to nrow(xp);
             do i=1 to nrow(x);
                 dist=sqrt((x[i]-xp[j])**2+(y[i]-yp[j])**2);
                 sgamma[i,j]=(c0+ce*(1-exp(-alpha*dist)));
             end;
        end;
        sgamma=sgamma//(xmatp`);
    finish;

/* Part 5: Defining the main IML module "krige", in which the kriging equations are solved */

    start krige;
        one=j(nrow(xmat),1,1);
        xmat=one||xmat;
        bgamma=j(nrow(x),nrow(x),0);
        sgamma=j(nrow(x),nrow(xp),0);
        one=j(nrow(xmatp),1,1);
        xmatp=one||xmatp;
        zero=j(ncol(xmat),1,0);
        z=z//zero;
        run gam1;
        run gam2;
        lambda=inv(bgamma)*sgamma;
        zhatm=lambda`*z;
            do i=1 to nrow(xp);
                 x0=xp[i];
                 y0=yp[i];
                 zhat=zhatm[i];
                 append;
            end;
   finish;

/* Part 6: Defining data set and parameter values; Actual execution ("run krige") of part 5 */

   use wetland;
         read all var{x} into x;
         read all var{y} into y;
         read all var{x} into xmat;
         read all var{prism} into z;
   use points;
         read all var{x0} into xp;
         read all var{y0} into yp;
         read all var{x0} into xmatp;
   C0=3179512663;     /* best fitting value from "semivari.lst" */
   Ce=6357843246;     /* best fitting value from "semivari.lst" */
   alpha=0.057475;     /* best fitting value from "semivari.lst" */
run krige;

/* Part 7: Writing the predicted values from the IML set "pred" to the output file "prism.nov" */

data a;
   set pred;
   zhats=zhat/1000;   /* to convert units to "x10^3" */
   file 'prism nov a';
         put x0 7.1 y0 7.1 zhats 12.0;
run;
